@@ -78,19 +78,19 @@
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static char sccsid[] = "@(#)gethostnamadr.c	8.1 (Berkeley) 6/4/93";
-static char rcsid[] = "$Id: gethnamaddr.c,v 1.6 2002/06/13 01:04:37 majka Exp $";
+static char rcsid[] = "$Id: gethnamaddr.c,v 1.8 2003/07/03 21:56:17 majka Exp $";
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <arpa/nameser.h>
+#include <arpa/nameser8_compat.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <netdb.h>
-#include <resolv.h>
+#include <resolv8_compat.h>
 #include <ctype.h>
 #include <errno.h>
 #include <syslog.h>
@@ -442,25 +442,6 @@ gethostbyname(name)
 	return (gethostbyname2(name, AF_INET));
 }
 
-struct hostent *
-gethostbyname2(name, af)
-	const char *name;
-	int af;
-{
-	if (_lu_running())
-	{
-		return getipnodebyname(name, af, 0, &h_errno);
-	}
-	else
-	{
-		if (af == AF_INET) return gethostbyname_ipv4(name);
-
-		errno = EAFNOSUPPORT;
-		h_errno = NETDB_INTERNAL;
-		return NULL;
-	}
-}
-
 static struct hostent *
 gethostbyname_ipv4(name)
 	const char *name;
