@@ -32,13 +32,12 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <stdio.h>
-#include <arpa/nameser.h>
-#include <resolv.h>
+#include <arpa/nameser8_compat.h>
+#include <resolv8_compat.h>
 
 /* From res_init.c */
 
 struct __res_state _res = {0};
-__private_extern__ struct __res_state _res_shadow = {0};
 
 /* From getnetent.c */
 
