@@ -46,7 +46,7 @@
 #include <sys/stat.h>
 #include <sys/mount.h>
 
-static FILE *_fs_fp;
+static FILE *_fs_fp = NULL;
 static struct fstab _fs_fstab;
 static struct fstab _root_fstab;
 static char _root_fstype[MFSNAMELEN];
@@ -181,7 +181,7 @@ static int fstabscan()
 	}
 
         for (;;) {
-		if (!(cp = fgets(line, sizeof(line), _fs_fp)))
+		if (!(cp = fgets(line, MAXLINELENGTH, _fs_fp)))
 			return(0);
 /* OLD_STYLE_FSTAB */
 		if (!strpbrk(cp, " \t")) {
@@ -191,11 +191,11 @@ static int fstabscan()
 				continue;
 #endif
 			_fs_fstab.fs_file = strtok((char *)NULL, ":\n");
-                        /* Only list the root filesystem once */
-                        if (!(strcmp(_fs_fstab.fs_file, "/"))) {
-                            continue;
-                        }
-                        _fs_fstab.fs_type = strtok((char *)NULL, ":\n");
+			/* Only list the root filesystem once */
+			if (!_fs_fstab.fs_file || !(strcmp(_fs_fstab.fs_file, "/"))) {
+				continue;
+			}
+			_fs_fstab.fs_type = strtok((char *)NULL, ":\n");
 			if (_fs_fstab.fs_type) {
 				if (!strcmp(_fs_fstab.fs_type, FSTAB_XX))
 					continue;
