@@ -53,7 +53,7 @@
 #if defined(LIBC_SCCS) && !defined(lint)
 /*static char *sccsid = "from: @(#)xdr_mem.c 1.19 87/08/11 Copyr 1984 Sun Micro";*/
 /*static char *sccsid = "from: @(#)xdr_mem.c	2.1 88/07/29 4.0 RPCSRC";*/
-static char *rcsid = "$Id: xdr_mem.c,v 1.3 2002/02/19 20:36:26 epeyton Exp $";
+static char *rcsid = "$Id: xdr_mem.c,v 1.5 2003/06/23 17:24:59 majka Exp $";
 #endif
 
 /*
@@ -104,7 +104,7 @@ xdrmem_create(xdrs, addr, size, op)
 	void *addr;
 #else
 	caddr_t addr;
-#endif /* !NeXT */
+#endif
 	u_int size;
 	enum xdr_op op;
 {
@@ -116,8 +116,7 @@ xdrmem_create(xdrs, addr, size, op)
 }
 
 static void
-xdrmem_destroy(/*xdrs*/)
-	/*XDR *xdrs;*/
+xdrmem_destroy(XDR *xdrs)
 {
 }
 
@@ -126,7 +125,6 @@ xdrmem_getlong(xdrs, lp)
 	register XDR *xdrs;
 	long *lp;
 {
-
 	if ((xdrs->x_handy -= sizeof(long)) < 0)
 		return (FALSE);
 	*lp = (long)ntohl((u_long)(*((long *)(xdrs->x_private))));
@@ -139,7 +137,6 @@ xdrmem_putlong(xdrs, lp)
 	register XDR *xdrs;
 	long *lp;
 {
-
 	if ((xdrs->x_handy -= sizeof(long)) < 0)
 		return (FALSE);
 	*(long *)xdrs->x_private = (long)htonl((u_long)(*lp));
@@ -179,7 +176,6 @@ static u_int
 xdrmem_getpos(xdrs)
 	register XDR *xdrs;
 {
-
 	return ((u_int)xdrs->x_private - (u_int)xdrs->x_base);
 }
 
